#include "mex.h"
#include "ANN.h"

// This is the MEX implementation of the MATLAB function 'annkdtree_k_nearest_neighbors'.
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    // Verify the correctness of the number of input and output parameters as well as their type:
    if(nlhs < 1 || 2 < nlhs || nrhs != 4 || !mxIsSingle(prhs[1])) mexErrMsgTxt("Incorrect Sintax\n\t[idxs, distSqrds] = annkdtree_k_nearest_neighbors(tree, p, k, eps), where 'p' is a SINGLE PRECISION vector\n");

    // Get pointer to the input kd-tree:
    double* pointer0 = mxGetPr(prhs[0]);
    long    pointer1 = (long) pointer0[0];
    ANNkd_tree* tree = (ANNkd_tree*) pointer1;
    
    // Get pointer to query data point and its dimension:
    float* queryDataPoint = (float*) mxGetData(prhs[1]);
    int dims = mxGetNumberOfElements(prhs[1]);
    
    int K      = (int) mxGetScalar(prhs[2]);
    double eps =       mxGetScalar(prhs[3]);

	ANNpoint queryPt = annAllocPt(dims);
    for(int j=0; j<dims; j++) queryPt[j] = queryDataPoint[j];
    
	ANNidxArray  nnIdx = new ANNidx[K];
	ANNdistArray dists = new ANNdist[K];
    
    // Search for the nearest neighbors:
    if(!tree->annkSearch(queryPt, K, nnIdx, dists, eps))
    {
        // If there was some error, return empty arrays:
        plhs[0] = mxCreateDoubleMatrix(1,0,mxREAL);
        if(nlhs == 2) plhs[1] = mxCreateDoubleMatrix(1,0,mxREAL);
    }
    
    // Allocate and populate the output array of nearest neighbor indices:
    plhs[0] = mxCreateDoubleMatrix(K, 1, mxREAL);
    double* indices = mxGetPr(plhs[0]);
    for(int k=0; k<K; k++) indices[k] = nnIdx[k] + 1;
    
    // If "distsSqrd" was required as an output:
    if(nlhs == 2)
    {
        // Allocate and populate the output array of nearest neighbor squared distances:
        plhs[1] = mxCreateDoubleMatrix(K, 1, mxREAL);
        double* distSqrds = mxGetPr(plhs[1]);
        for(int k=0; k<K; k++) distSqrds[k] = dists[k];
    }
    
    delete[] nnIdx;
    delete[] dists;
}
